@@ -1,16 +1,24 @@
 using Gtk;
 using Gdk;
 using System;
+using Mono.Addins;
 using Mono.Unix;
 
+using FSpot.Extensions;
+using FSpot.Utils;
+using FSpot.UI.Dialog;
+using FSpot.Widgets;
+using FSpot.Platform;
+
 namespace FSpot {
 	public class SingleView {
 		[Glade.Widget] Gtk.HBox toolbar_hbox;
 		[Glade.Widget] Gtk.VBox info_vbox;
 		[Glade.Widget] Gtk.ScrolledWindow image_scrolled;
-		[Glade.Widget] Gtk.ScrolledWindow directory_scrolled;
 		[Glade.Widget] Gtk.HPaned info_hpaned;
 
+		Gtk.ScrolledWindow directory_scrolled;
+
 		[Glade.Widget] Gtk.CheckMenuItem side_pane_item;
 		[Glade.Widget] Gtk.CheckMenuItem toolbar_item;
 		[Glade.Widget] Gtk.CheckMenuItem filenames_item;
@@ -24,14 +32,19 @@ namespace FSpot {
 
 		[Glade.Widget] Label status_label;
 
+		[Glade.Widget] ImageMenuItem rotate_left;
+		[Glade.Widget] ImageMenuItem rotate_right;
+
+		ToolButton rr_button, rl_button;
+
+		Sidebar sidebar;
+
 		protected Glade.XML xml;
 		private Gtk.Window window;
 		PhotoImageView image_view;
-		IconView directory_view;
+		FSpot.Widgets.IconView directory_view;
 		private Uri uri;
 		
-		InfoDialog metadata_dialog;
-		
 		UriCollection collection;
 		
 		FullScreenView fsview;
@@ -41,7 +54,7 @@ namespace FSpot {
 		public SingleView () : this (FSpot.Global.HomeDirectory) {}
 
 
-		public SingleView (string path) : this (UriList.PathToFileUri (path)) 
+		public SingleView (string path) : this (UriUtils.PathToFileUri (path)) 
 		{
 		}
 
@@ -70,12 +83,12 @@ namespace FSpot {
 			Gtk.Toolbar toolbar = new Gtk.Toolbar ();
 			toolbar_hbox.PackStart (toolbar);
 		
-			ToolButton rl_button = GtkUtil.ToolButtonFromTheme ("object-rotate-left", Catalog.GetString ("Rotate Left"), true);
+			rl_button = GtkUtil.ToolButtonFromTheme ("object-rotate-left", Catalog.GetString ("Rotate Left"), true);
 			rl_button.Clicked += HandleRotate270Command;
 			rl_button.SetTooltip (toolTips, Catalog.GetString ("Rotate photo left"), null);
 			toolbar.Insert (rl_button, -1);
 
-			ToolButton rr_button = GtkUtil.ToolButtonFromTheme ("object-rotate-right", Catalog.GetString ("Rotate Right"), true);
+			rr_button = GtkUtil.ToolButtonFromTheme ("object-rotate-right", Catalog.GetString ("Rotate Right"), true);
 			rr_button.Clicked += HandleRotate90Command;
 			rr_button.SetTooltip (toolTips, Catalog.GetString ("Rotate photo right"), null);
 			toolbar.Insert (rr_button, -1);
@@ -94,23 +107,40 @@ namespace FSpot {
 
 			collection = new UriCollection (uris);
 
-			TargetEntry [] dest_table = {   new TargetEntry ("text/uri-list", 0, 0),
-							new TargetEntry ("text/plain", 0, 1)};
+			TargetEntry [] dest_table = {
+				FSpot.DragDropTargets.UriListEntry,
+				FSpot.DragDropTargets.PlainTextEntry
+			};
 			
-			directory_view = new IconView (collection);
+			directory_view = new FSpot.Widgets.IconView (collection);
 			directory_view.Selection.Changed += HandleSelectionChanged;
 			directory_view.DragDataReceived += HandleDragDataReceived;
 			Gtk.Drag.DestSet (directory_view, DestDefaults.All, dest_table, 
 					DragAction.Copy | DragAction.Move); 
 			directory_view.DisplayTags = false;
 			directory_view.DisplayDates = false;
+			directory_view.DisplayRatings = false;
+
+			directory_scrolled = new ScrolledWindow();
 			directory_scrolled.Add (directory_view);
 
+			sidebar = new Sidebar ();
+
+			info_vbox.Add (sidebar);
+			sidebar.AppendPage (directory_scrolled, Catalog.GetString ("Folder"), "gtk-directory");
+
+			AddinManager.AddExtensionNodeHandler ("/FSpot/Sidebar", OnSidebarExtensionChanged);
+ 		
+			sidebar.Context = ViewContext.Single;
+
+			sidebar.CloseRequested += HandleHideSidePane;
+			sidebar.Show ();
+
 			ThumbnailGenerator.Default.OnPixbufLoaded += delegate { directory_view.QueueDraw (); };
 
 			image_view = new PhotoImageView (collection);
-			FSpot.Global.ModifyColors (image_view);
-			FSpot.Global.ModifyColors (image_scrolled);
+			GtkUtil.ModifyColors (image_view);
+			GtkUtil.ModifyColors (image_scrolled);
 			image_view.ZoomChanged += HandleZoomChanged;
 			image_view.Item.Changed += HandleItemChanged;
 			image_view.ButtonPressEvent += HandleImageViewButtonPressEvent;
@@ -136,16 +166,35 @@ namespace FSpot {
 			window.DeleteEvent += HandleDeleteEvent;
 			
 			collection.Changed += HandleCollectionChanged;
+
+			// wrap the methods to fit to the delegate
+			image_view.Item.Changed += delegate (object sender, BrowsablePointerChangedEventArgs old) {
+					BrowsablePointer pointer = sender as BrowsablePointer;
+					if (pointer == null)
+						return;
+					IBrowsableItem [] item = {pointer.Current};
+					PhotoArray item_array = new PhotoArray (item);
+					sidebar.HandleSelectionChanged (item_array);
+			};
+			
+			image_view.Item.Collection.ItemsChanged += sidebar.HandleSelectionItemsChanged;
+
 			UpdateStatusLabel ();
 			
 			if (collection.Count > 0)
 				directory_view.Selection.Add (0);
 
-			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
+			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem (this).Submenu;
 			export.Submenu.ShowAll ();
 			export.Activated += HandleExportActivated ;
 		}
 
+		private void OnSidebarExtensionChanged (object s, ExtensionNodeEventArgs args) {
+			// FIXME: No sidebar page removal yet!
+			if (args.Change == ExtensionChange.Add)
+				sidebar.AppendPage ((args.ExtensionNode as SidebarPageNode).GetSidebarPage ());
+		}
+
 		void HandleExportActivated (object o, EventArgs e)
 		{
 			FSpot.Extensions.ExportMenuItemNode.SelectedImages = delegate () {return new FSpot.PhotoArray (directory_view.Selection.Items); };
@@ -161,6 +210,8 @@ namespace FSpot {
 			if (collection.Count > 1)
 				ShowSidebar = true;
 
+			rotate_left.Sensitive = rotate_right.Sensitive = rr_button.Sensitive = rl_button.Sensitive = collection.Count != 0;
+
 			UpdateStatusLabel ();
 		}
 
@@ -201,34 +252,34 @@ namespace FSpot {
 		void HandleRotate90Command (object sender, System.EventArgs args) 
 		{
 			RotateCommand command = new RotateCommand (this.Window);
-			if (command.Execute (RotateDirection.Clockwise, new IBrowsableItem [] { image_view.Item.Current })) {
-				collection.MarkChanged (image_view.Item.Index);
-			}
+			if (command.Execute (RotateDirection.Clockwise, new IBrowsableItem [] { image_view.Item.Current }))
+				collection.MarkChanged (image_view.Item.Index, FullInvalidate.Instance);
 		}
 
 		void HandleRotate270Command (object sender, System.EventArgs args) 
 		{
 			RotateCommand command = new RotateCommand (this.Window);
-			if (command.Execute (RotateDirection.Counterclockwise, new IBrowsableItem [] { image_view.Item.Current })) {
-				collection.MarkChanged (image_view.Item.Index);
-			}
+			if (command.Execute (RotateDirection.Counterclockwise, new IBrowsableItem [] { image_view.Item.Current }))
+				collection.MarkChanged (image_view.Item.Index, FullInvalidate.Instance);
 		}		
 
 		private void HandleSelectionChanged (FSpot.IBrowsableCollection selection) 
 		{
 			
 			if (selection.Count > 0) {
-				image_view.Item.Index = ((IconView.SelectionCollection)selection).Ids[0];
+				image_view.Item.Index = ((FSpot.Widgets.IconView.SelectionCollection)selection).Ids[0];
 
 				zoom_scale.Value = image_view.NormalizedZoom;
-				if (metadata_dialog != null)
-					metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
 			}
 			UpdateStatusLabel ();
 		}
 
-		private void HandleItemChanged (BrowsablePointer pointer, BrowsablePointerChangedArgs old)
+		private void HandleItemChanged (object sender, BrowsablePointerChangedEventArgs old)
 		{
+			BrowsablePointer pointer = sender as BrowsablePointer;
+			if (pointer == null)
+				return;
+
 			directory_view.FocusCell = pointer.Index;
 			directory_view.Selection.Clear ();
 			if (collection.Count > 0) {
@@ -244,7 +295,7 @@ namespace FSpot {
 			if (current == null)
 				return;
 
-			Preferences.SetAsBackground (current.DefaultVersionUri.LocalPath);
+			Desktop.SetBackgroundImage (current.DefaultVersionUri.LocalPath);
 		}
 
 		private void HandleViewToolbar (object sender, System.EventArgs args)
@@ -264,31 +315,10 @@ namespace FSpot {
 
 		private void HandleViewSlideshow (object sender, System.EventArgs args)
 		{
-			if (fsview != null)
-				fsview.Destroy ();
-
-			fsview = new FSpot.FullScreenView (collection);
-			fsview.Destroyed += HandleFullScreenViewDestroy;
-
-			fsview.View.Item.Index = image_view.Item.Index;
-			fsview.Show ();
+			HandleViewFullscreen (sender, args);
 			fsview.PlayPause ();
 		}
 	
-		private void HandleViewMetadata (object sender, System.EventArgs args)
-		{
-			if (metadata_dialog != null) {
-				metadata_dialog.Present ();
-				return;
-			}
-			
-			metadata_dialog = new InfoDialog (window);
-			metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
-			
-			metadata_dialog.ShowAll ();
-			metadata_dialog.Destroyed += HandleMetadataDestroyed;
-		}
-
 		private void HandleViewFilenames (object sender, System.EventArgs args)
 		{
 			directory_view.DisplayFilenames = filenames_item.Active; 
@@ -297,7 +327,7 @@ namespace FSpot {
 
 		private void HandleAbout (object sender, System.EventArgs args)
 		{
-			MainWindow.HandleAbout (sender, args);
+			FSpot.UI.Dialog.AboutDialog.ShowUp ();
 		}
 
 		private void HandleNewWindow (object sender, System.EventArgs args)
@@ -345,11 +375,6 @@ namespace FSpot {
 			chooser.Destroy ();
 		}
 
-		private void HandleMetadataDestroyed (object sender, System.EventArgs args)
-		{
-			metadata_dialog = null;
-		}
-	
 		private bool SlideShow ()
 		{
 			IBrowsableItem [] items = new IBrowsableItem [collection.Count];
@@ -368,7 +393,7 @@ namespace FSpot {
 			if (fsview != null)
 				fsview.Destroy ();
 
-			fsview = new FSpot.FullScreenView (collection);
+			fsview = new FSpot.FullScreenView (collection, window);
 			fsview.Destroyed += HandleFullScreenViewDestroy;
 
 			fsview.View.Item.Index = image_view.Item.Index;
@@ -442,23 +467,22 @@ namespace FSpot {
 
 		void HandleDragDataReceived (object sender, DragDataReceivedArgs args) 
 		{
-		
-		switch (args.Info) {
-		case 0:
-		case 1:
-			/* 
-			 * If the drop is coming from inside f-spot then we don't want to import 
-			 */
-			if (Gtk.Drag.GetSourceWidget (args.Context) != null)
+			if (args.Info == FSpot.DragDropTargets.UriListEntry.Info
+			    || args.Info == FSpot.DragDropTargets.PlainTextEntry.Info) {
+				
+				/* 
+				 * If the drop is coming from inside f-spot then we don't want to import 
+				 */
+				if (Gtk.Drag.GetSourceWidget (args.Context) != null)
+					return;
+				
+				UriList list = args.SelectionData.GetUriListData ();
+				collection.LoadItems (list.ToArray());
+				
+				Gtk.Drag.Finish (args.Context, true, false, args.Time);
+				
 				return;
-
-			UriList list = new UriList (args.SelectionData);
-			collection.LoadItems (list.ToArray());
-
-			break;
-		}
-
-		Gtk.Drag.Finish (args.Context, true, false, args.Time);
+			}
 		}
 
 		private void UpdateStatusLabel ()
@@ -514,21 +538,16 @@ namespace FSpot {
 			file_selector.Destroy ();
 		}
 
-		void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
+		void OnPreferencesChanged (object sender, NotifyEventArgs args)
 		{
 			LoadPreference (args.Key);
 		}
 
 		void LoadPreference (String key)
 		{
-			object val = Preferences.Get (key);
-
-			if (val == null)
-				return;
-			
 			switch (key) {
 			case Preferences.VIEWER_MAXIMIZED:
-				if ((bool) val)
+				if (Preferences.Get<bool> (key))
 					window.Maximize ();
 				else
 					window.Unmaximize ();
@@ -536,43 +555,43 @@ namespace FSpot {
 
 			case Preferences.VIEWER_WIDTH:
 			case Preferences.VIEWER_HEIGHT:
-				window.SetDefaultSize((int) Preferences.Get(Preferences.VIEWER_WIDTH),
-						(int) Preferences.Get(Preferences.VIEWER_HEIGHT));
+				window.SetDefaultSize(Preferences.Get<int> (Preferences.VIEWER_WIDTH),
+						      Preferences.Get<int> (Preferences.VIEWER_HEIGHT));
 
 				window.ReshowWithInitialSize();
 				break;
 			
 			case Preferences.VIEWER_SHOW_TOOLBAR:
-				if (toolbar_item.Active != (bool) val)
-					toolbar_item.Active = (bool) val;
+				if (toolbar_item.Active != Preferences.Get<bool> (key))
+					toolbar_item.Active = Preferences.Get<bool> (key);
 
-				toolbar_hbox.Visible = (bool) val;
+				toolbar_hbox.Visible = Preferences.Get<bool> (key);
 				break;
 
 			case Preferences.VIEWER_INTERPOLATION:
-				if ((bool) val)
+				if (Preferences.Get<bool> (key))
 					image_view.Interpolation = Gdk.InterpType.Bilinear;
 				else
 					image_view.Interpolation = Gdk.InterpType.Nearest;
 				break;
 
 			case Preferences.VIEWER_SHOW_FILENAMES:
-				if (filenames_item.Active != (bool) val)
-					filenames_item.Active = (bool) val;
+				if (filenames_item.Active != Preferences.Get<bool> (key))
+					filenames_item.Active = Preferences.Get<bool> (key);
 				break;
 
 			case Preferences.VIEWER_TRANSPARENCY:
-				if (val as string == "CHECK_PATTERN")
-					image_view.SetCheckSize (2);
-				else if (val as string == "COLOR")
-					image_view.SetTransparentColor (Preferences.Get(Preferences.VIEWER_TRANS_COLOR) as string);
+				if (Preferences.Get<string> (key) == "CHECK_PATTERN")
+					image_view.CheckPattern = CheckPattern.Dark;
+				else if (Preferences.Get<string> (key) == "COLOR")
+					image_view.CheckPattern = new CheckPattern (Preferences.Get<string> (Preferences.VIEWER_TRANS_COLOR));
 				else // NONE
-					image_view.SetTransparentColor (image_view.Style.BaseColors [(int)Gtk.StateType.Normal]);
+					image_view.CheckPattern = new CheckPattern (image_view.Style.BaseColors [(int)Gtk.StateType.Normal]);
 				break;
 
 			case Preferences.VIEWER_TRANS_COLOR:
-				if (Preferences.Get(Preferences.VIEWER_TRANSPARENCY) as string == "COLOR")
-					image_view.SetTransparentColor (val as string);
+				if (Preferences.Get<string> (Preferences.VIEWER_TRANSPARENCY) == "COLOR")
+					image_view.CheckPattern = new CheckPattern (Preferences.Get<string> (key));
 				break;
 			}
 		}
@@ -632,7 +651,7 @@ namespace FSpot {
 				prefs.Dialog.Present ();
 			}
 
-			void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
+			void OnPreferencesChanged (object sender, NotifyEventArgs args)
 			{
 				LoadPreference (args.Key);
 			}
@@ -649,17 +668,13 @@ namespace FSpot {
 
 			void LoadPreference (string key)
 			{
-				object val = Preferences.Get (key);
-
-				if (val == null)
-					return;
-			
+		
 				switch (key) {
 				case Preferences.VIEWER_INTERPOLATION:
-					interpolation_check.Active = (bool) val;
+					interpolation_check.Active = Preferences.Get<bool> (key);
 					break;
 				case Preferences.VIEWER_TRANSPARENCY:
-					switch ((string) val) {
+					switch (Preferences.Get<string> (key)) {
 					case "COLOR":
 						as_color_radio.Active = true;
 						break;
@@ -673,9 +688,9 @@ namespace FSpot {
 					break;
 				case Preferences.VIEWER_TRANS_COLOR:
 					color_button.Color = new Gdk.Color (
-						Byte.Parse ((val as string).Substring (1,2), System.Globalization.NumberStyles.AllowHexSpecifier),
-						Byte.Parse ((val as string).Substring (3,2), System.Globalization.NumberStyles.AllowHexSpecifier),
-						Byte.Parse ((val as string).Substring (5,2), System.Globalization.NumberStyles.AllowHexSpecifier));
+						Byte.Parse (Preferences.Get<string> (key).Substring (1,2), System.Globalization.NumberStyles.AllowHexSpecifier),
+						Byte.Parse (Preferences.Get<string> (key).Substring (3,2), System.Globalization.NumberStyles.AllowHexSpecifier),
+						Byte.Parse (Preferences.Get<string> (key).Substring (5,2), System.Globalization.NumberStyles.AllowHexSpecifier));
 					break;
 				}
 			}
